from datetime import datetime
from importlib.metadata import version
import sys

from log import CustomLogger

from drive.utilities.parser import generate_cmd_parser


def main() -> None:
    parser = generate_cmd_parser()

    args = parser.parse_args()

    # getting the programs start time
    start_time = datetime.now()

    # creating and configuring the logger and then recording user inputs
    logger = CustomLogger.create_logger()

    if not hasattr(args, "output"):
        logger.configure("", None, args.verbose, args.log_to_console)
    elif args.output.is_dir():
        logger.configure(
            args.output, args.log_filename, args.verbose, args.log_to_console
        )
    else:
        logger.configure(
            args.output.parent, args.log_filename, args.verbose, args.log_to_console
        )

    logger.info(f"DRIVE version: {version('drive-ibd')}")

    logger.info(f"command passed to DRIVE: {' '.join(sys.argv)}")

    # record the input parameters using a method from the logger object that
    # takes the parser as an argument
    logger.record_namespace(args)
    # We need to record the output directory when we run DRIVE unless its for unit test
    if hasattr(args, "output"):
        logger.debug(f"Parent directory for log files and output: {args.output.parent}")

    logger.info(f"Analysis start time: {start_time}")

    args.func(args)

    end_time = datetime.now()

    logger.info(
        f"Analysis finished at {end_time}. Total runtime: {end_time - start_time}"
    )


if __name__ == "__main__":
    main()



import argparse
import pytest
from pathlib import Path
import sysconfig
import sys

from log import CustomLogger

logger = CustomLogger.get_logger(__name__)


def run_integration_test(args: argparse.Namespace) -> None:
    logger.info("Running integration test to ensure that DRIVE was installed correctly")

    site_packages_path = Path(sysconfig.get_paths().get("platlib")) / "tests"

    if site_packages_path is None:
        logger.fatal(
            "Was not able to find the site-packages directory in the virtualenv that DRIVE was installed into. This error is unexpected. Please report this error to the maintainers. Terminating program..."
        )
        sys.exit(1)

    sys.path.append(site_packages_path)

    pytest.main(["-v", site_packages_path / "test_integration.py"])



from dataclasses import dataclass, field
from pathlib import Path
from typing import Any, Callable, Dict, List, Optional, Union

import matplotlib.pyplot as plt
import numpy.typing as npt
import pandas as pd
import scipy.cluster.hierarchy as sch
from log import CustomLogger
from numpy import zeros
from scipy.spatial import distance
from scipy.spatial.distance import squareform

from drive.network.filters import IbdFilter
from drive.network.models import create_indices
from drive.utilities.functions import split_target_string

logger = CustomLogger.get_logger(__name__)


class NetworkIDNotFound(Exception):
    def __init__(self, network_id: str) -> None:
        self.msg = f"The id, {network_id}, was not found in the IBD data. Make sure that the ID you are looking for perfectly matches what should be in the DRIVE file. Trailing zeros can make a difference, 10.0 != 10 in DRIVE."
        super().__init__(self.msg)


def check_kwargs(args_dict: dict[str, Any]) -> Optional[str]:
    """Function that will make sure that the necessary arguments are passed to distance function

    Parameters
    ----------
    args_dict : Dict[str, Any]
        Dictionary that has the arguments as keys and the values for the distance function
    """
    if not all(
        [
            elem in args_dict.keys()
            for elem in ["pair_1", "pair_2", "pairs_df", "cm_threshold"]
        ]
    ):
        return "Not enough arguments passed to the _determine_distances function. \
            Expected pair_1, pair_2, pairs_df, cm_threshold"


def _determine_distances(**kwargs) -> tuple[Optional[str], float]:
    """Function that will determine the distances between the main grid and then \
        each connected grid. It will use a value of 2.5 for all grids that don't \
            share a segment. This is just the min cM value, 5cM, divided in half

    Parameters
    ----------
    pair_1 : str
        string that has the main grid id


    pairs_df : pd.DataFrame
        dataframe from the pairs file that has been filtered to just connections \
            with the main grid
    """
    # checking to make sure the function has the right parameters in the kwargs and then
    # returning an error if it doesn't
    err = check_kwargs(kwargs)

    if err is not None:
        return err, 0.0

    # getting all the necessary values out of the kwargs dict
    pair_1 = kwargs.pop("pair_1")
    pair_2 = kwargs.pop("pair_2")
    pairs_df = kwargs.pop("pairs_df")
    min_cM = kwargs.pop("cm_threshold")

    if pair_2 == pair_1:
        return None, float(0)

    # pulling out the length of the IBD segment based on hapibd
    filtered_pairs: pd.DataFrame = pairs_df[
        ((pairs_df["pair_1"] == pair_1) & (pairs_df["pair_2"] == pair_2))
        | ((pairs_df["pair_1"] == pair_2) & (pairs_df["pair_2"] == pair_1))
    ]

    if filtered_pairs.empty:
        ibd_length: float = min_cM / 2

    else:
        ibd_length: float = filtered_pairs.iloc[0]["length"]

    return None, 1 / ibd_length


def record_matrix(output: Union[Path, str], matrix, pair_list: List[str]) -> None:
    """Function that will write the distance matrix to a file

    Parameters
    ----------
    output : str
        filepath to write the output to

    matrix : array
        array that has the distance matrix for each individual

    pair_list : List[str]
        list of ids that represent each row of the pair_list
    """

    with open(output, "w") as output_file:
        for i in range(len(pair_list)):
            distance_str = "\t".join([str(distance) for distance in matrix[i]])
            output_file.write(f"{pair_list[i]}\t{distance_str}\n")


def map_network_ids(id_list: List[str]) -> Dict[str, str]:
    """Map the IDs from the original network to random patient ids (Ex: IDA -> Patient_1).
    This is mainly useful for publication

    Parameters
    ----------
    id_list : List[str]
        list of strings representing the people in the network that the
        distance matrix will be calculated for.

    Returns
    -------
    Dict[str, str]
        Returns a list where the original ids are the keys and the strings they were mapped
            :withto 'Patient_1, Patient_2, etc...' are values"
    """

    return {
        original_id: f"patient_{indx}"
        for indx, original_id in enumerate(id_list, start=1)
    }


@dataclass
class DistanceMatrixResults:
    distance_matrix_id_mapping: Union[List[str], None]
    distance_matrix: npt.NDArray
    id_mapping: Union[Dict[str, str], None] = field(default_factory=dict)


def make_distance_matrix(
    pairs_df: pd.DataFrame,
    min_cM: int,
    map_ids: bool,
    distance_function: Callable = _determine_distances,
) -> DistanceMatrixResults:
    """Function that will make the distance matrix

    Parameters
    ----------
    pairs_df : pd.DataFrame
        dataframe that has the pairs_files. it should have at least three columns
        called 'pair_1', 'pair_2', and 'length'

    min_cM : float
        This is the minimum centimorgan threshold that will be divided in half to
        get the ibd segment length when pairs do not share a segment

    map_ids : bool
        Boolean indication whether we want to map the ids to an random mapping or not

    Returns
    -------
    Dict[str, Dict[str, float]]
        returns a tuple where the first object is a list of ids that has the
        individual id that corresponds to each row. The second object is the
        distance matrix
    """
    # get a list of all the unique ids in the pairs_df
    id_list = list(
        set(pairs_df.pair_1.values.tolist() + pairs_df.pair_2.values.tolist())
    )

    if map_ids:
        # dictionary that has the mappign from the original id to the new ids
        id_mapping = map_network_ids(id_list)

        # We need to save the new ids as the id_list for later code to work out
        id_list = list(id_mapping.values())
    else:
        id_mapping = None

    matrix = zeros((len(id_list), len(id_list)), dtype=float)

    ids_index = []

    for i in range(len(id_list)):
        ids_index.append(id_list[i])

        for j in range(len(id_list)):
            err, distance = distance_function(
                pair_1=id_list[i],
                pair_2=id_list[j],
                pairs_df=pairs_df,
                cm_threshold=min_cM,
            )
            if err is not None:
                print(err)
                return DistanceMatrixResults(None, None, {})

            matrix[i][j] = distance

    return DistanceMatrixResults(ids_index, matrix, id_mapping)


def generate_dendrogram(matrix: npt.NDArray) -> npt.NDArray:
    """Function that will perform the hierarchical clustering algorithm

    Parameters
    ----------
    matrix : Array
        distance matrix represented by 2D numpy array.
        distance should be calculated based on 1/(ibd segment
        length)

    Returns
    -------
    Array
        returns the results of the clustering as a numpy array
    """
    squareform_matrix = squareform(matrix)

    return sch.linkage(squareform_matrix, method="ward")


# def _check_for_overlap(cases: List[str], exclusions: List[str]) -> None:
#     """Make sure there is no overlap between individuals in the cases and exclusions \
#         list provided to the _generate_label_colors method

#     Parameters
#     ----------
#     cases : List[str]
#         list of individuals who are considered cases for a
#         disease or phenotype

#     exclusions : List[str]
#         list of individuals who are consider exclusions and are indicated as N/A or
#         -1 by the phenotype file. This value defaults to None.

#     Raises
#     ------
#     ValueError
#         If there are overlapping individuals in the cases and exclusions lists then a \
#             ValueError is raised to indicate that the user should check their labelling.
#     """
#     if [ind for ind in cases if ind in exclusions]:
#         raise ValueError(
#             "Overlapping individuals found between cases and exclusions \
#                         lists. Please check your case and exclusions list labelling."
#         )


# def _generate_label_colors(
#     grid_list: List[str], cases: List[str], exclusions: List[str] = []
# ) -> Dict[str, str]:
#     """Function that will generate the color dictionary
#     indicating what color each id label should be

#     Parameters
#     ----------
#     grid_list : list[str]
#         list of id strings

#     cases : List[str]
#         list of individuals who are considered cases for a
#         disease or phenotype

#     exclusions : List[str]
#         list of individuals who are consider exclusions and are indicated as N/A or
#         -1 by the phenotype file. This value defaults to None

#     Returns
#     -------
#     dict[str,str]
#         returns a dictionary where the key is the id and the
#         values are the color of the label for that id.
#     """

#     if exclusions:
#         _check_for_overlap(cases, exclusions)

#     color_dict = {}

#     for grid in grid_list:
#         if grid in cases:
#             color_dict[grid] = "r"
#         elif grid in exclusions:
#             color_dict[grid] = "g"
#         else:
#             color_dict[grid] = "k"

#     return color_dict


def draw_dendrogram(
    clustering_results: npt.NDArray,
    grids: List[str],
    output_name: Union[Path, str],
    # cases: Optional[List[str]] = None,
    # exclusions: List[str] = [],
    title: Optional[str] = None,
    node_font_size: int = 10,
) -> tuple[plt.Figure, plt.Axes, Dict[str, Any]]:
    """Function that will draw the dendrogram

    Parameters
    ----------
    clustering_results : npt.NDArray
        numpy array that has the results from running the generate_dendrogram function

    grids : list[str]
        list of ids to use as labels

    output_name : Path | str
        path object or a string that tells where the
        dendrogram will be saved to.

    # cases : list[str] | None
    #     list of case ids. If the user doesn't provided this
    #     value then all of the labels on the dendrogram will
    #     be black. If the user provides a value then the case
    #     labels will be red. Value defaults to None

    # exclusions : List[str]
    #     list of individuals who are consider exclusions and are
    #     indicated as N/A or -1 by the phenotype file. This value
    #     defaults to None

    title : str | None
        Optional title for the plot. If this is not provided
        then the plot will have no title

    node_font_size : int
        Size for the font of the dendrogram leaf nodes

    Returns
    -------
    tuple[plt.Figure, plt.Axes, dict[str, Any]]
        returns a tuple with the matplotlib Figure, the
        matplotlib Axes object, and a dictionary from the sch.
        dendrogram command
    """
    figure = plt.figure(figsize=(15, 12))
    ax = plt.subplot(111)

    # Temporarily override the default line width:
    with plt.rc_context(
        {
            "lines.linewidth": 2,
        }
    ):
        dendrogram = sch.dendrogram(
            clustering_results,
            labels=grids,
            orientation="left",
            color_threshold=0,
            above_threshold_color="black",
            leaf_font_size=node_font_size,
        )

    # change the color of the nodes for cases if the user wants to.

    # # remove whitespace from the case labels
    # cases = [case.strip() for case in cases]

    # if cases:
    #     color_dict = _generate_label_colors(grids, cases, exclusions)
    #     yaxis_labels = ax.get_ymajorticklabels()
    #     for label in yaxis_labels:
    #         label.set_color(color_dict[label.get_text()])

    # removing the ticks and axes
    ax.spines["right"].set_visible(False)
    ax.spines["top"].set_visible(False)
    ax.spines["left"].set_visible(False)
    ax.spines["bottom"].set_visible(False)
    ax.axes.get_xaxis().set_visible(False)

    if title:
        plt.title(title, fontsize=20)

    plt.savefig(output_name)

    return figure, ax, dendrogram


def load_networks(
    drive_results: Path,
    max_network_size: int,
    min_network_size: int,
    network_id: str = Optional[str],
) -> dict[str, List[str]]:
    """method reads the drive networks into a dictionary. If the user chooses to
    generate dendrograms for all the networks then DRIVe only returns those
    networks that are between a certain size range

    Parameters
    ----------
    drive_results : Path
        output file from running the clustering step of DRIVE

    max_network_size : int
        the maximum size of a network that will be allowed in the analysis.
        Drawing large networks well is hard to automate. The default value is 30

    min_network_size : int
        minimum size of a network to draw. Drawing 2 person networks makes no
        sense so by default DRIVE only draws dendrograms for networks of 3 or
        more people. Users can choose a different value if they wish.

    network_id : str
        the id of the cluster ID by DRIVE. Default value = none

    Returns
    -------
    dict[str, list[str]]
        returns a dictionary where the keys are the cluster IDs and the values
        are a list of haplotypes in the cluster

    Raises
    ------
    NetworkIDNotFount
        raises a KeyError if the network_id value is not found in the drive file
    """
    return_dict = {}

    with open(drive_results, "r", encoding="utf-8") as drive_file:
        logger.verbose(
            f"Identifying network {network_id} within the DRIVE output, {drive_results}"
        )
        if network_id:
            for line in drive_file:
                (
                    clst_id,
                    network_size,
                    haplotype_count,
                    edge_count,
                    connectedness,
                    _,
                    id_list,
                    haplotype_list,
                    *_,
                ) = line.strip().split("\t")
                if clst_id == network_id:
                    return_dict[clst_id] = {
                        "ids": id_list.strip().split(","),
                        "haplotypes": haplotype_list.strip().split(","),
                    }

            if not return_dict:
                logger.critical(
                    f"The id {network_id} was not found within the DRIVE file at {drive_file}."
                )
                raise NetworkIDNotFound(network_id)
        else:
            logger.verbose(
                "No network id passed to program. Reading in data for all of the networks identified by DRIVE."
            )
            logger.verbose(
                f"For the sake of drawing efficiency, only gathering dendrograms that are between size {min_network_size} and {max_network_size}"
            )

            for line in drive_file:
                # we can skip the header line
                if "clstID" in line:
                    pass
                else:
                    (
                        clst_id,
                        network_size,
                        haplotype_count,
                        edge_count,
                        connectedness,
                        _,
                        id_list,
                        haplotype_list,
                        *_,
                    ) = line.strip().split("\t")
                    if min_network_size <= len(id_list.split(",")) <= max_network_size:
                        return_dict[clst_id] = {
                            "ids": id_list.strip().split(","),
                            "haplotypes": haplotype_list.strip().split(","),
                        }

    logger.debug(f"Stored {len(return_dict.keys())} networks within the dictionary")

    return return_dict


def generate_dendrograms(args) -> None:
    # need to add read in the drive networks to get the appropriate ids
    network_ids = load_networks(
        args.input, args.max_network_size, args.min_network_size, args.network_id
    )
    # generate an object that has all of the indices for the correct ibd format
    indices = create_indices(args.format.lower())

    ##target gene region or variant position
    target_gene = split_target_string(args.target)

    for clstID, id_dict in network_ids.items():
        # generate output path for the dendrogram image
        logger.info(f"generating dendrogram for network {clstID}")
        args.output.mkdir(parents=True, exist_ok=True)

        full_output_path = args.output / f"network_{clstID}_dendrogram.png"

        id_list = id_dict["ids"]
        haplotype_list = id_dict["haplotypes"]

        filter_obj: IbdFilter = IbdFilter.load_file(args.ibd, indices, target_gene)

        # choosing the proper way to filter the ibd files
        filter_obj.set_filter(args.segment_overlap)
        # Filter the IBD data to only the sites that were used in the DRIVE analysis
        filter_obj.preprocess(args.min_cm, id_list)

        ibd_segments = filter_obj.ibd_pd
        print(ibd_segments)

        # We need to make sure that the we are look at only the
        # right haplotypes
        haplotype_filtered_ibd_segments = ibd_segments[
            (ibd_segments.hapid1.isin(haplotype_list))
            & (ibd_segments.hapid2.isin(haplotype_list))
        ]

        haplotype_filtered_ibd_segments = haplotype_filtered_ibd_segments.rename(
            columns={
                indices.id1_indx: "pair_1",
                indices.id2_indx: "pair_2",
                indices.cM_indx: "length",
            }
        )

        # matrix_id_list, distance_matrix = make_distance_matrix(
        #     haplotype_filtered_ibd_segments[["pair_1", "pair_2", "length"]],
        #     args.min_cm,
        #     args.map_ids,
        # )
        distanceMatrixObj = make_distance_matrix(
            haplotype_filtered_ibd_segments[["pair_1", "pair_2", "length"]],
            args.min_cm,
            args.map_ids,
        )

        if distanceMatrixObj.id_mapping:
            with open(
                args.output / f"network_{clstID}_id_mapping.txt", "w", encoding="utf-8"
            ) as mapping_fh:
                mapping_fh.write("Original_ID\tMapped_ID\n")
                for orig_id, new_id in distanceMatrixObj.id_mapping.items():
                    mapping_fh.write(f"{orig_id}\t{new_id}\n")

        if args.keep_temp:
            temp_dir = args.output / f"network_{clstID}_temp"
            temp_dir.mkdir(parents=True, exist_ok=True)

            full_output_matrix_path = temp_dir / f"network_{clstID}_distance_matrix.txt"

            logger.info(
                f"recorded the distance matrix to the following path: {full_output_matrix_path}"
            )

            record_matrix(
                full_output_matrix_path,
                distanceMatrixObj.distance_matrix,
                distanceMatrixObj.distance_matrix_id_mapping,
            )

        dendrogram_array = generate_dendrogram(distanceMatrixObj.distance_matrix)

        logger.info(f"writing the output dendrogram to {full_output_path}")

        _ = draw_dendrogram(
            dendrogram_array,
            distanceMatrixObj.distance_matrix_id_mapping,
            full_output_path,
            args.title,
            args.font_size,
        )



import sys
from log import CustomLogger

logger = CustomLogger.get_logger(__name__)


def map_header_cols_to_indx(header_line: str) -> dict[str, int]:
    """We want to map column in the header to an index so that later we can access the right column"""
    return {
        col_name: indx for indx, col_name in enumerate(header_line.strip().split("\t"))
    }


def run_pull_samples(args) -> None:
    # We need to make sure the correct arguments were passed
    if not args.cases_only and args.case_col is not None:
        logger.fatal(
            "Expected the case-col flag to be empty since the case-only flag was not used. If you intend to only pull cases please pass the cases-only flag"
        )
        sys.exit(1)
    if args.cases_only and args.case_col is None:
        logger.fatal(
            "Expect arguments to be passed to the case-col flag since the cases-only flag was used. If you intend to pull only cases please provide both the cases-only flag and the column name to pull using the case-col flag."
        )

    with open(args.input, "r", encoding="utf-8") as drive_results:
        samples = []
        # map the header line to an index
        header_map = map_header_cols_to_indx(next(drive_results))

        # If the user only wants to pull the column then we have to find
        # the index for that column header. Otherwise we can just pull
        # the IDs column for the whole network
        if args.cases_only:
            try:
                col_indx = header_map[args.case_col]
            except KeyError:
                logger.fatal(
                    f"Could not find the column, {args.case_col}, in the header. Please make sure you spelled the column you wish to grab exactly as it appears in the header"
                )
                sys.exit(1)
        else:
            col_indx = 6

        for line in drive_results:
            split_line = line.strip().split("\t")
            if split_line[0] == args.network_id:
                samples = split_line[col_indx].strip().split(",")
                break
        if len(samples) != 0:
            with open(args.output, "w", encoding="utf-8") as output_file:
                for grid in samples:
                    output_file.write(f"{grid.strip()}\n")
        else:
            logger.critical(
                f"The network id, {args.network_id}, was not found within the networks file at {args.input}. Please ensure that all of the network ids are properly formatted and that the network is present within your input file."
            )



from collections import namedtuple
from pathlib import Path
import pandas as pd
import pytest
import sys

sys.path.append("./drive")

from drive.filters import IbdFilter
from drive.models.generate_indices import HapIBD

hapibd = HapIBD()

Genes = namedtuple("Genes", ["chr", "start", "end"])


@pytest.mark.unit
def test_ibd_file_not_found() -> None:
    """Unit test that will make sure a FileNotFound Exception is raised if the ibd file doesn't exist"""
    fake_gene = Genes(12, 1235, 1235)
    with pytest.raises(FileNotFoundError):
        IbdFilter.load_file(Path("FileDoesNotExist.txt"), hapibd, fake_gene)


@pytest.mark.kcne1
def test_filter_results() -> None:
    """Unit test that will make sure that the Filter class produces the proper results"""

    true_ibdpd = pd.read_csv("tests/test_true_results/ibdpd_KNCE1.txt", sep="\t")

    kcne1_gene = Genes(21, 35818986, 35884508)

    filter_obj = IbdFilter.load_file(
        Path("tests/test_inputs/biovu_longQT_EUR_chr21.ibd.gz"), hapibd, kcne1_gene
    )

    kcne1_min_cm_thres = 3

    filter_obj.preprocess(kcne1_min_cm_thres)

    error_list = []

    # Make sure that the code identifies the correct number of IBD segments as before
    if filter_obj.ibd_pd.shape[0] != true_ibdpd.shape[0]:
        error_list.append(
            f"The filtered dataframe from the filter_obj is not the same length as the file test ibdpd_KCNE1.txt. Expected the file to have {true_ibdpd.shape[0]} instead there were {filter_obj.ibd_pd.shape[0]}"
        )

    # Make sure that the program identifies the same number of unique haplotypes as before
    max_id_new = filter_obj.ibd_pd[["idnum1", "idnum2"]].values.max()
    max_id_old = true_ibdpd[["idnum1", "idnum2"]].values.max()

    if max_id_new != max_id_old:
        error_list.append(
            f"The maximum haplotype id assigned in the new version is not the same as before. Expected value to be {max_id_old}, instead it was {max_id_new}"
        )

    assert not error_list, "errors occured:\n{}".format("\n".join(error_list))



import itertools
import pytest
import sys
import pandas as pd
import sysconfig
from pathlib import Path

sys.path.append("./src")

from drive import drive


site_packages_path = Path(sysconfig.get_paths().get("platlib"))


# @pytest.mark.integtest
# def test_drive_full_run():
#     assert 1==1
@pytest.fixture()
def system_args_no_pheno(monkeypatch):
    input_file = (
        site_packages_path / "tests/test_inputs/simulated_ibd_test_data_v2_chr20.ibd.gz"
    )

    output_file = (
        site_packages_path / "tests/test_output/integration_test_results_no_pheno"
    )

    monkeypatch.setattr(
        "sys.argv",
        [
            "drive",
            "cluster",
            "-i",
            str(input_file.absolute()),
            "-f",
            "hapibd",
            "-t",
            "20:4666882-4682236",
            "-o",
            str(output_file.absolute()),
            "-m",
            "3",
            "--recluster",
            "--log-filename",
            "integration_test_results_no_pheno.log",
        ],
    )


@pytest.fixture()
def system_args_with_pheno(monkeypatch):
    input_file = (
        site_packages_path / "tests/test_inputs/simulated_ibd_test_data_v2_chr20.ibd.gz"
    )

    output_file = (
        site_packages_path / "tests/test_output/integration_test_results_with_pheno"
    )

    carrier_file = (
        site_packages_path / "tests/test_inputs/test_phenotype_file_withNAs.txt"
    )

    monkeypatch.setattr(
        "sys.argv",
        [
            "drive",
            "cluster",
            "-i",
            str(input_file.absolute()),
            "-f",
            "hapibd",
            "-t",
            "20:4666882-4682236",
            "-o",
            str(output_file.absolute()),
            "-m",
            "3",
            "-c",
            str(carrier_file.absolute()),
            "--recluster",
            "--log-file",
            "integration_test_results_with_pheno.log",
        ],
    )


@pytest.fixture()
def system_args_for_dendrogram(monkeypatch):
    input_file = (
        site_packages_path
        / "tests/test_inputs/integration_dendrogram_test_results_no_pheno.drive_networks.txt"
    )

    output_path = site_packages_path / "tests/test_output/"

    ibd_file = (
        site_packages_path / "tests/test_inputs/simulated_ibd_test_data_v2_chr20.ibd.gz"
    )

    monkeypatch.setattr(
        "sys.argv",
        [
            "drive",
            "dendrogram",
            "-i",
            str(input_file.absolute()),
            "-f",
            "hapibd",
            "-t",
            "20:4666882-4682236",
            "-o",
            str(output_path.absolute()),
            "-n",
            "0",
            "-m",
            "3",
            "--ibd",
            str(ibd_file.absolute()),
            "--log-file",
            "integration_dendrogram_test_results.log",
        ],
    )


@pytest.mark.integtest
def test_drive_full_run_no_phenotypes(system_args_no_pheno):
    # Make sure the output directory exists
    output_path = site_packages_path / "tests/test_output"
    output_path.mkdir(exist_ok=True)

    drive.main()

    # we need to make sure the output was properly formed
    output = pd.read_csv(
        site_packages_path
        / "tests/test_output/integration_test_results_no_pheno.drive_networks.txt",
        sep="\t",
    )
    # list of errors to keep
    errors = []

    # list of columns it should have
    expected_colnames = [
        "clstID",
        "n.total",
        "n.haplotype",
        "true.positive.n",
        "true.positive",
        "falst.postive",
        "IDs",
        "ID.haplotype",
    ]

    if not output.shape == (165, 8):
        errors.append(
            f"Expected the output to have 165 rows and 8 columns instead it had {output.shape[0]} rows and {output.shape[1]}"
        )
    if [col for col in output.columns if col not in expected_colnames]:
        errors.append(
            f"Expected the output to have the columns: {','.join(expected_colnames)}, instead these columns were found: {','.join(output.columns)}"
        )

    assert not errors, "errors occured:\n{}".format("\n".join(errors))


@pytest.mark.integtest
def test_drive_full_run_with_phenotypes(system_args_with_pheno):
    # Make sure the output directory exists
    output_path = site_packages_path / "tests/test_output"
    output_path.mkdir(exist_ok=True)

    drive.main()

    # we need to make sure the output was properly formed
    output = pd.read_csv(
        site_packages_path
        / "tests/test_output/integration_test_results_with_pheno.drive_networks.txt",
        sep="\t",
    )

    # list of errors to keep
    errors = []

    # lets read in the header of the phenotype file so that we can form the additional columns
    with open(
        site_packages_path / "tests/test_inputs/test_phenotype_file_withNAs.txt", "r"
    ) as pheno_input:
        grid_col, pheno1, pheno2, pheno3 = pheno_input.readline().strip().split("\t")

        col_combinations = list(
            itertools.product(
                [pheno1, pheno2, pheno3],
                [
                    "_case_count_in_network",
                    "_cases_in_network",
                    "_excluded_count_in_network",
                    "_excluded_in_network",
                    "_pvalue",
                ],
            )
        )

        phenotype_cols = [
            "min_pvalue",
            "min_phenotype",
            "min_phenotype_description",
        ] + ["".join(val) for val in col_combinations]

    # list of columns it should have
    expected_colnames = [
        "clstID",
        "n.total",
        "n.haplotype",
        "true.positive.n",
        "true.positive",
        "falst.postive",
        "IDs",
        "ID.haplotype",
    ] + phenotype_cols

    if not output.shape == (165, 26):
        errors.append(
            f"Expected the output to have 165 rows and 8 columns instead it had {output.shape[0]} rows and {output.shape[1]}"
        )
    if [col for col in output.columns if col not in expected_colnames]:
        errors.append(
            f"Expected the output to have the columns: {','.join(expected_colnames)}, instead these columns were found: {','.join(output.columns)}"
        )

    assert not errors, "errors occured:\n{}".format("\n".join(errors))


@pytest.mark.integtest
def test_drive_dendrogram_single_network(system_args_for_dendrogram):
    output_path = site_packages_path / "tests/test_output"
    output_path.mkdir(exist_ok=True)

    drive.main()

    output_path = output_path / "network_0_dendrogram.png"

    assert output_path.exists(), (
        "An error occurred while running the integration test for the dendrogram functionality. This error prevented the appropriate output from being generated."
    )


@pytest.fixture()
def system_args_for_pull_samples(monkeypatch):
    input_path = (
        site_packages_path
        / "tests/test_inputs/integration_dendrogram_test_results_no_pheno.drive_networks.txt"
    )
    output_path = site_packages_path / "tests/test_output/test_sample_list.txt"

    monkeypatch.setattr(
        "sys.argv",
        [
            "drive",
            "utilities",
            "pull-samples",
            "-i",
            str(input_path.absolute()),
            "-o",
            str(output_path.absolute()),
            "-n",
            "4",
        ],
    )


@pytest.mark.integtest
def test_pull_samples_success(system_args_for_pull_samples):
    output_path = site_packages_path / "tests/test_output"
    output_path.mkdir(exist_ok=True)

    # run the drive function
    drive.main()

    samples_filepath = output_path / "test_sample_list.txt"

    assert samples_filepath.exists(), (
        f"An error occurred while running the integration test for the utilties 'pull-samples' subcommand. the output file, {samples_filepath}, was not found."
    )


@pytest.mark.integtest
def test_for_correct_samples(system_args_for_pull_samples):
    output_path = site_packages_path / "tests/test_output"
    output_path.mkdir(exist_ok=True)

    # run the drive function
    drive.main()

    samples_filepath = output_path / "test_sample_list.txt"

    # we are going to make a set of samples to look for and use this to check if all of the samples are in the file
    samples_to_find = {"535", "574", "94", "210", "676"}

    samples_in_file = set()

    with open(samples_filepath, "r") as samples_file:
        # we are making the assumption that each line in the file is a single id
        for line in samples_file:
            samples_in_file.add(line.strip())

    sample_differences = samples_to_find.difference(samples_in_file)

    assert not sample_differences, (
        f"The samples, {','.join(sample_differences)}, were expected to be found within the file but were not. Instead, only these values were found, {','.join(samples_in_file)}"
    )



import pytest
import sys

sys.path.append("./drive")

from drive.utilities.parser import PhenotypeFileParser


@pytest.mark.unit
def test_file_not_found() -> None:
    """Test that the parsers raises a FileNotFoundError if the phenotype file doesn't exist."""
    with pytest.raises(FileNotFoundError):
        with PhenotypeFileParser("./NotRealFile.txt") as pheno_file:
            ...


@pytest.mark.unit
@pytest.mark.parametrize(
    "test_str,expected", [("ID1\t1", "\t"), ("ID1,1", ","), ("ID1|0", "|")]
)
def test_check_separator(test_str: str, expected: str) -> None:
    """Test that the method _check_separator to see if it returns proper separator."""

    assert PhenotypeFileParser._check_separator(test_str) == expected


@pytest.mark.unit
@pytest.mark.parametrize("test_str", [("ID1\n1")])
def test_check_separator_error(test_str: str) -> None:
    """Test that the method _check_separator raises a ValueError if it cannot identify the separator."""
    with pytest.raises(ValueError):
        PhenotypeFileParser._check_separator(test_str)


@pytest.mark.unit
def test_single_phenotype_output_keys() -> None:
    """Check that the PhenotypeFileParser identifies the correct phenotype in the output."""
    with PhenotypeFileParser("./tests/test_inputs/test_phenotype_file.txt") as parser:
        phenotype_counts, _ = parser.parse_cases_and_controls()

        assert "status" in phenotype_counts.keys()


@pytest.mark.unit
def test_single_phenotype_output_key_length() -> None:
    """Check that the PhenotypeFileParser identifies the correct number of phenotypes in the output."""
    with PhenotypeFileParser("./tests/test_inputs/test_phenotype_file.txt") as parser:
        phenotype_counts, _ = parser.parse_cases_and_controls()

        assert len(phenotype_counts.keys()) == 1


@pytest.mark.unit
def test_case_control_exclusion_counts() -> None:
    """Check that the PhenotypeFileParser identifies the correct case/control/exlcusion counts."""
    with PhenotypeFileParser("./tests/test_inputs/test_phenotype_file.txt") as parser:
        phenotype_counts, _ = parser.parse_cases_and_controls()

        cases = phenotype_counts["status"]["cases"]
        controls = phenotype_counts["status"]["controls"]
        exclusions = phenotype_counts["status"]["excluded"]

        expected_case_counts = 8
        expected_controls_counts = 84
        expected_exclusion_counts = 8

        error_list = []

        if len(cases) != expected_case_counts:
            error_list.append(
                f"Expected parser to identify {expected_case_counts} cases. Instead {len(cases)} cases were identified."
            )
        if len(controls) != expected_controls_counts:
            error_list.append(
                f"Expected parser to identify {expected_controls_counts} cases. Instead {len(controls)} cases were identified"
            )
        if len(exclusions) != expected_exclusion_counts:
            error_list.append(
                f"Expected parser to identify {expected_exclusion_counts} cases. Instead {len(exclusions)} cases were identified"
            )

        assert not error_list, "errors occured:\n{}".format("\n".join(error_list))


@pytest.mark.unit
def test_multiple_phenotype_key_counts() -> None:
    """Check that the PhenotypeFileParser output identifies the correct number of phenotypes."""
    with PhenotypeFileParser(
        "./tests/test_inputs/test_multiple_phenotype_file.txt"
    ) as parser:
        phenotype_counts, _ = parser.parse_cases_and_controls()

        assert len(phenotype_counts.keys()) == 2


@pytest.mark.unit
def test_multiple_phenotype_counts() -> None:
    """Check that the PhenotypeFileParser identifies the correct case/control/exlcusion counts for multiple phenotypes."""
    with PhenotypeFileParser(
        "./tests/test_inputs/test_multiple_phenotype_file.txt"
    ) as parser:
        phenotype_counts, _ = parser.parse_cases_and_controls()

        cases_status_1 = phenotype_counts["status_1"]["cases"]
        controls_status_1 = phenotype_counts["status_1"]["controls"]
        exclusions_status_1 = phenotype_counts["status_1"]["excluded"]

        cases_status_2 = phenotype_counts["status_2"]["cases"]
        controls_status_2 = phenotype_counts["status_2"]["controls"]
        exclusions_status_2 = phenotype_counts["status_2"]["excluded"]

        expected_case_counts_status_1 = 8
        expected_controls_counts_status_1 = 84
        expected_exclusion_counts_status_1 = 8

        expected_case_counts_status_2 = 7
        expected_controls_counts_status_2 = 88
        expected_exclusion_counts_status_2 = 5

        error_list = []

        if len(cases_status_1) != expected_case_counts_status_1:
            error_list.append(
                f"Expected parser to identify {expected_case_counts_status_1} cases. Instead {len(cases_status_1)} cases were identified."
            )

        if len(controls_status_1) != expected_controls_counts_status_1:
            error_list.append(
                f"Expected parser to identify {expected_controls_counts_status_1} cases. Instead {len(controls_status_1)} cases were identified"
            )

        if len(exclusions_status_1) != expected_exclusion_counts_status_1:
            error_list.append(
                f"Expected parser to identify {expected_exclusion_counts_status_1} cases. Instead {len(exclusions_status_1)} cases were identified"
            )

        if len(cases_status_2) != expected_case_counts_status_2:
            error_list.append(
                f"Expected parser to identify {expected_case_counts_status_2} cases. Instead {len(cases_status_2)} cases were identified."
            )

        if len(controls_status_2) != expected_controls_counts_status_2:
            error_list.append(
                f"Expected parser to identify {expected_controls_counts_status_2} cases. Instead {len(controls_status_2)} cases were identified"
            )

        if len(exclusions_status_2) != expected_exclusion_counts_status_2:
            error_list.append(
                f"Expected parser to identify {expected_exclusion_counts_status_2} cases. Instead {len(exclusions_status_2)} cases were identified"
            )

        assert not error_list, "errors occured:\n{}".format("\n".join(error_list))



# isort: skip_file
from .case_file_parser import PhenotypeFileParser
from .phenotype_descriptions_parser import load_phenotype_descriptions
from .cmdline_parser import (
    generate_cmd_parser,
)  # This import needs to come after the load_phenotype_descriptions to avoid a circular import. Isort would change this which is why this file is skipped



import argparse
from importlib.metadata import version
from pathlib import Path

from rich_argparse import RichHelpFormatter

from drive.dendrogram import generate_dendrograms
from drive.network import run_network_identification
from drive.utilities.pull_samples import run_pull_samples
from drive.utilities.callbacks import CheckInputExist
from drive.utilities.testing import run_integration_test


def generate_cmd_parser() -> argparse.ArgumentParser:
    """Function that will generate the correct cmd parser for DRIVE"""
    parser = argparse.ArgumentParser(
        description=" Distant Relatedness for Identification and Variant Evaluation (DRIVE) is a novel approach to IBD-based genotype inference used to identify shared chromosomal segments in dense genetic arrays",
        formatter_class=RichHelpFormatter,
    )

    parser.add_argument(
        "--version", action="version", version=f"%(prog)s: {version('drive-ibd')}"
    )

    # We are also going to create a parser for common options
    common_parser = argparse.ArgumentParser(
        formatter_class=RichHelpFormatter, add_help=False
    )

    logging_group = common_parser.add_argument_group(
        "logging",
        description="parameters that affect what runtime information is printed to a file or the console",
    )

    logging_group.add_argument(
        "--verbose",
        "-v",
        default=0,
        help="verbose flag indicating if the user wants more information",
        action="count",
    )

    logging_group.add_argument(
        "--log-to-console",
        default=False,
        help="Optional flag to log to only the console or also a file",
        action="store_true",
    )

    logging_group.add_argument(
        "--log-filename",
        default="drive.log",
        type=str,
        help="Name for the log output file. (default: %(default)s)",
    )

    # This first section will handle all of the arguments for the
    # clustering subcommand
    subparser = parser.add_subparsers(
        title="subcommands",
        description="options to run either the DRIVE clustering algorithm or the dendrogram algorithm. To run DRIVE, type either: 'drive cluster --help' or 'drive dendrogram --help'",
    )

    cluster_parser = subparser.add_parser(
        name="cluster",
        help="run the DRIVE clustering algorithm",
        formatter_class=RichHelpFormatter,
        parents=[common_parser],
        description="cluster",
    )

    cluster_parser.add_argument(
        "--input",
        "-i",
        type=Path,
        help="IBD input file from ibd detection software",
        required=True,
        action=CheckInputExist,
    )

    cluster_parser.add_argument(
        "--format",
        "-f",
        default="hapibd",
        type=str,
        help="IBD program used to detect segments. Allowed values are hapibd, ilash, germline, rapid. Program expects for value to be lowercase. (default: %(default)s)",
        choices=["hapibd", "ilash", "germline", "rapid"],
    )

    cluster_parser.add_argument(
        "--target",
        "-t",
        type=str,
        help="Target region or position, chr:start-end or chr:pos",
        required=True,
    )

    cluster_parser.add_argument(
        "--output",
        "-o",
        type=Path,
        help="output file prefix. The program will append .drive_networks.txt to the filename provided",
        required=True,
    )

    cluster_parser.add_argument(
        "--min-cm",
        "-m",
        default=3,
        type=int,
        help="minimum centimorgan threshold. The program expects this to be an integer value. (default: %(default)s)",
    )

    cluster_parser.add_argument(
        "--step",
        "-k",
        default=3,
        type=int,
        help="Minimum required number of steps for the community walktrap algorithm.(default: %(default)s)",
    )

    cluster_parser.add_argument(
        "--max-recheck",
        default=5,
        type=int,
        help="Maximum number of times to re-perform the clustering. This value will not be used if the flag --no-recluster is used.(default: %(default)s)",  # noqa: E501
    )

    cluster_parser.add_argument(
        "--cases",
        "-c",
        type=Path,
        help="A file containing individuals who are cases. This file expects for there to be two columns. The first column will have individual ids and the second has status where cases are indicated by a 1, control are indicated by a 0, and exclusions are indicated by NA.",  # noqa: E501
        action=CheckInputExist,
    )

    cluster_parser.add_argument(
        "--segment-overlap",
        default="contains",
        choices=["contains", "overlaps"],
        type=str,
        help="Indicates if the user wants the gene to contain the whole target region or if it just needs to overlap the segment. (default: %(default)s)",  # noqa: E501
    )

    cluster_parser.add_argument(
        "--descriptions",
        "-d",
        type=Path,
        help="tab delimited text file that has descriptions for each phecode. this file should have two columns called phecode and phenotype",  # noqa: E501
    )

    cluster_parser.add_argument(
        "--max-network-size",
        default=30,
        type=int,
        help="maximum network size allowed if the user has allowed the recluster option. (default: %(default)s)",
    )

    cluster_parser.add_argument(
        "--min-connected-threshold",
        default=0.5,
        type=float,
        help="minimum connectedness ratio required for the network. (default: %(default)s)",
    )

    cluster_parser.add_argument(
        "--min-network-size",
        default=3,
        type=int,
        help="This argument sets the minimun network size that we allow. All networks smaller than this size will be filtered out. If the user wishes to keep all networks they can set this to 0. (default: %(default)s)",  # noqa: E501
    )

    cluster_parser.add_argument(
        "--segment-distribution-threshold",
        default=0.2,
        type=float,
        help="Threshold to filter the network length to remove hub individuals. (default: %(default)s)",
    )

    cluster_parser.add_argument(
        "--phenotype-name",
        default=None,
        type=str,
        help="If the user wishes to only run 1 phenotype from a file with multiple phenotypes they can prioritize a column that has the phenotype name. The name must match with the column.",
    )

    cluster_parser.add_argument(
        "--hub-threshold",
        default=0.01,
        type=float,
        help="Threshold to determine what percentage of hubs to keep. (default: %(default)s)",
    )

    cluster_parser.add_argument(
        "--json-config",
        "-j",
        default=None,
        type=Path,
        help="path to the json config file",
    )

    cluster_parser.add_argument(
        "--recluster",
        type=bool,
        default=True,
        action=argparse.BooleanOptionalAction,
        help="whether or not the user wishes the program to automically recluster based on things like hub threshold, max network size and how connected the graph is. ",  # noqa: E501
    )

    cluster_parser.add_argument(
        "--chunksize",
        type=int,
        default=100_000,
        help="change the chunksize used to read in the shared segment data. Larger chunksizes will speed up the analysis but will use more memory. There is a asymptotic limit on the speed up still. Due to how pandas reads in data, trying to read in the whole file at once will still be slower than chunking if the file is really big. (default: %(default)s)",
    )

    cluster_parser.set_defaults(func=run_network_identification)

    # This is where we will define all of the necessary arguments to make
    # the dendrogram script work
    dendrogram_parser = subparser.add_parser(
        name="dendrogram",
        help="run the DRIVE dendrogram program",
        formatter_class=RichHelpFormatter,
        parents=[common_parser],
        description="dendrogram",
    )

    dendrogram_parser.add_argument(
        "--input",
        "-i",
        type=Path,
        help="Input file to generate the dendrogram from. This file should be the output from running DRIVE",
        required=True,
        action=CheckInputExist,
    )

    dendrogram_parser.add_argument(
        "--ibd",
        type=Path,
        help="path to the input ibd file that was used to create the networks from",
        required=True,
        action=CheckInputExist,
    )

    dendrogram_parser.add_argument(
        "--output",
        "-o",
        help="directory to write output to. Default value: %(default)s",
        default=Path("./"),
        type=Path,
    )

    dendrogram_parser.add_argument(
        "--keep-temp",
        default=False,
        help="Optional flag to retain the intermediate distance matrices. (default: %(default)s)",
        action="store_true",
    )

    dendrogram_parser.add_argument(
        "--format",
        "-f",
        default="hapibd",
        type=str,
        help="IBD program used to detect segments. Allowed values are hapibd, ilash, germline, rapid. Program expects for value to be lowercase. (default: %(default)s)",
        choices=["hapibd", "ilash", "germline", "rapid"],
    )

    dendrogram_parser.add_argument(
        "--target",
        "-t",
        type=str,
        help="Target region or position, chr:start-end or chr:pos",
        required=True,
    )

    dendrogram_parser.add_argument(
        "--segment-overlap",
        default="contains",
        choices=["contains", "overlaps"],
        type=str,
        help="Indicates if the user wants the gene to contain the whole target region or if it just needs to overlap the segment. (default: %(default)s)",  # noqa: E501
    )

    dendrogram_parser.add_argument(
        "--min-cm",
        "-m",
        default=3,
        type=int,
        help="minimum centimorgan threshold. The program expects this to be an integer value. (default: %(default)s)",
    )

    dendrogram_parser.add_argument(
        "--map-ids",
        default=False,
        help="Map the ids in the network to an anonymous ID of the form patient_X. This labelling is mainly used for publication to change internal identifiers. The mapping will be saved in the output directory as the dendrogram as a file called 'network_{clstID}_id_mappings.txt' (default: %(default)s)",
        action="store_true",
    )

    # Add a mutually exclusive group that requires you to either provide the
    # argument for the network id or the generate-all flag
    exclusive_group = dendrogram_parser.add_mutually_exclusive_group(required=True)

    exclusive_group.add_argument(
        "--network-id",
        "-n",
        type=str,
        help="Network ID to make dendrograms only for a specific cluster. This value needs to match what is in the clstID column from the output from DRIVE.",
    )

    exclusive_group.add_argument(
        "--generate-all",
        default=False,
        help="Optional flag to choose to generate dendrograms for all networks. This option will require longer runtime. (default: %(default)s)",
        action="store_true",
    )

    dendrogram_parser.add_argument(
        "--max-network-size",
        type=int,
        default=30,
        help="maximum network size to make a dendrogram for. When networks are really large they are hard to visualize. We suggest using your own script for these networks. This value will only be used if the user chooses to generate all the dendrograms in the DRIVE output. (default: %(default)s)",
    )

    dendrogram_parser.add_argument(
        "--min-network-size",
        type=int,
        default=3,
        help="minimum network size to make a dendrogram for. By default, DRIVE can return 2 person networks. Dendrogram should not be made for these because they are uninformative. Users can also select different values if they are interested in returning networks of only size 'x' or larger. This value will only be used if the user chooses to generate all the dendrograms in the DRIVE output. (default: %(default)s)",
    )

    dendrogram_parser.add_argument(
        "--font-size",
        default=15,
        type=int,
        help="set the label size for the output dendrograms. (default: %(default)s)",
    )

    dendrogram_parser.add_argument(
        "--title",
        type=str,
        default="test dendrogram",
        help="title of the dendrogram to be written on the plot. This is not the output file name. This argument should not be used if you are creating dendrograms for every network provided by DRIVE",
    )

    dendrogram_parser.set_defaults(func=generate_dendrograms)

    # The utilities parser is going to be used to create help commands
    # that can process the file. This can be used as follows: drive utilities <subcommand> ...
    utilities_parser = subparser.add_parser(
        name="utilities",
        help="Run subcommands that help process the drive file",
        formatter_class=RichHelpFormatter,
        parents=[common_parser],
        description="utilities",
    )

    utility_cmd_subparser = utilities_parser.add_subparsers(
        title="utility commands",
        description="helper functions that help process the DRIVE file",
    )

    # This command will be used to pull samples from a network of interest and provide that to
    pull_samples_parser = utility_cmd_subparser.add_parser(
        name="pull-samples",
        help="pull sample ids from a network of interest",
        formatter_class=RichHelpFormatter,
        parents=[common_parser],
        description="pull-samples",
    )

    pull_samples_parser.add_argument(
        "-n",
        "--network-id",
        type=str,
        required=True,
        help="ID of the network to pull from the DRIVE results file",
    )

    pull_samples_parser.add_argument(
        "-i",
        "--input",
        type=Path,
        required=True,
        help="filepath to a tab separated file that described each of the network from the drive analysis. This command is designed to work with the output from the drive cluster command. You can use the output file from DRIVE as input or a file with the same columns and structure.",
    )

    pull_samples_parser.add_argument(
        "--cases-only",
        default=False,
        help="Optional flag that indicates if the user wants to pull the ids for all samples in a network or just the cases. If this flag is provided then the user also needs to provide the '--case-col' flag. (default: %(default)s)",
        action="store_true",
    )

    pull_samples_parser.add_argument(
        "--case-col",
        type=str,
        help="This is the column that list the cases in the network of interest. This flag should only be used if the '--cases-",
    )

    pull_samples_parser.add_argument(
        "-o",
        "--output",
        type=Path,
        default=Path("samples.txt"),
        help="Output tab separated text file with one column where each row is a sample id. This file is designed to work with bcftools. (default: %(default)s)",
    )

    pull_samples_parser.set_defaults(func=run_pull_samples)

    testing_parser = utility_cmd_subparser.add_parser(
        name="test",
        help="run the integration test to ensure that DRIVE was installed correctly",
        formatter_class=RichHelpFormatter,
        parents=[common_parser],
        description="run-integration-test",
    )

    testing_parser.add_argument(
        "--run-integration-test",
        default=True,
        help="Flag that indicates the user wishes to run the integration test. (default: %(default)s)",
        action="store_true",
    )

    testing_parser.set_defaults(func=run_integration_test)

    return parser



import re

from drive.network.models import Genes


def split_target_string(chromo_pos_str: str) -> Genes:
    """Function that will split the target string provided by the user.

    Parameters
    ----------
    chromo_pos_str : str
        String that has the region of interest in base pairs.
        This string will look like 10:1234-1234 where the
        first number is the chromosome number, then the start
        position, and then the end position of the region of
        interest.

    Returns
    -------
    Genes
        returns a namedtuple that has the chromosome number,
        the start position, and the end position

    Raises
    ------
    ValueError
        raises a value error if the string was formatted any
        other way than chromosome:start_position-end_position.
        Also raises a value error if the start position is
        larger than the end position
    """
    split_str = re.split(":|-", chromo_pos_str)

    if len(split_str) != 3:
        error_msg = f"Expected the gene position string to be formatted like chromosome:start_position-end_position. Instead it was formatted as {chromo_pos_str}"  # noqa: E501

        raise ValueError(error_msg)

    integer_split_str = [int(value) for value in split_str]

    if integer_split_str[1] > integer_split_str[2]:
        raise ValueError(
            f"expected the start position of the target string to be <= the end position. Instead the start position was {integer_split_str[1]} and the end position was {integer_split_str[2]}"  # noqa: E501
        )

    return Genes(*integer_split_str)



import argparse
import sys
from pathlib import Path


class CheckInputExist(argparse.Action):
    def __init__(self, option_strings, dest, nargs=None, **kwargs) -> None:
        if nargs is not None:
            raise ValueError("nargs not allowed")
        super(CheckInputExist, self).__init__(option_strings, dest, **kwargs)

    def __call__(
        self,
        parser: argparse.ArgumentParser,
        namespace: argparse.Namespace,
        values: Path,
        option_string: str = None,
    ) -> None:
        if values.exists():
            setattr(namespace, self.dest, values)
        else:
            print(
                f"ERROR: The file, {values}, was not found. Please make sure that there is not a typo in the file name."
            )
            sys.exit(1)



import json
import re
from pathlib import Path

from log import CustomLogger

import drive.network.factory as factory
from drive.network.cluster import ClusterHandler, cluster
from drive.network.filters import IbdFilter
from drive.network.models import Data, create_indices
from drive.utilities.functions import split_target_string
from drive.utilities.parser import PhenotypeFileParser, load_phenotype_descriptions

logger = CustomLogger.get_logger(__name__)


def find_json_file() -> Path:
    """Method to find the default config file if the user does not provide one

    Returns
    -------
    Path
        returns the path to the json file

    Raises
    ------
    FileNotFoundError
        Raises a FileNotFoundError if the program can not locate a json file and the
        user does not provide the path to a file
    """

    src_dir = Path(__file__).parent.parent

    config_path = src_dir / "config.json"

    if not config_path.exists():
        raise FileNotFoundError(
            f"Expected the user to either pass a configuration file path or for the config.json file to be present in the program root directory at {config_path}."  # noqa: E501
        )

    return config_path


def run_network_identification(args) -> None:
    """main entrypoint to run the clustering algorithm for DRIVE"""
    # We need to make sure that there is a configuration file
    json_config = args.json_config if args.json_config else find_json_file()

    # we need to load in the phenotype descriptions file to get
    # descriptions of each phenotype
    if args.descriptions:
        logger.verbose(f"Using the phenotype descriptions file at: {args.descriptions}")
        desc_dict = load_phenotype_descriptions(args.descriptions)
    else:
        logger.verbose("No phenotype descriptions provided")
        desc_dict = {}

    # if the user has provided a phenotype file then we will determine case/control/
    # exclusion counts. Otherwise we return an empty dictionary
    if args.cases:
        with PhenotypeFileParser(args.cases, args.phenotype_name) as phenotype_file:
            phenotype_counts, cohort_ids = phenotype_file.parse_cases_and_controls()

            logger.info(
                f"identified {len(phenotype_counts.keys())} phenotypes within the file {args.cases}"  # noqa: E501
            )
    else:
        logger.info(
            "No phenotype information provided. Only the clustering step of the analysis will be performed"  # noqa: E501
        )

        phenotype_counts = {}
        cohort_ids = {}

    indices = create_indices(args.format.lower())

    logger.debug(f"created indices object: {indices}")

    ##target gene region or variant position
    target_gene = split_target_string(args.target)

    logger.debug(f"Identified a target region: {target_gene}")

    filter_obj: IbdFilter = IbdFilter.load_file(
        args.input, indices, target_gene, args.chunksize
    )

    # choosing the proper way to filter the ibd files
    filter_obj.set_filter(args.segment_overlap)

    filter_obj.preprocess(args.min_cm, cohort_ids)

    # We need to invert the hapid_map dictionary so that the
    # integer mappings are keys and the values are the
    # haplotype string
    hapid_inverted = {value: key for key, value in filter_obj.hapid_map.items()}

    # creating the object that will handle clustering within the networks
    cluster_handler = ClusterHandler(
        args.min_connected_threshold,
        args.max_network_size,
        args.max_recheck,
        args.step,
        args.min_network_size,
        args.segment_distribution_threshold,
        args.hub_threshold,
        hapid_inverted,
        args.recluster,
    )

    networks = cluster(filter_obj, cluster_handler, indices.cM_indx)

    # creating the data container that all the plugins can interact with
    plugin_api = Data(networks, args.output, phenotype_counts, desc_dict)

    logger.debug(f"Data container: {plugin_api}")

    # making sure that the output directory is created
    # This section will load in the analysis plugins and run each plugin
    with open(json_config, encoding="utf-8") as json_config:
        config = json.load(json_config)

        factory.load_plugins(config["plugins"])

        analysis_plugins = [factory.factory_create(item) for item in config["modules"]]

        logger.debug(
            f"Using plugins: {', '.join([obj.name for obj in analysis_plugins])}"
        )

        # iterating over every plugin and then running the analyze and write method
        for analysis_obj in analysis_plugins:
            analysis_obj.analyze(data=plugin_api)
